package com.lightspeedleader.ui;

import javax.microedition.lcdui.*;
import java.util.Timer;
import java.util.TimerTask;

public class FlexiTicker extends CustomItem implements ItemCommandListener {

    private int width, height;

    //private Displayable displayable;

    private boolean ticking;

    private TickerTask tickerTask;

    private Timer timer;
    /**
     * The message set in this Ticker
     */
    private String message;

    /**
     * The message to display in this Ticker
     */
    private String displayedMessage;

    /**
     * The current location on screen of the scrolling message
     */
    private int messageLoc;

    /**
     * The pixel width of the message being displayed
     */
    private int messageWidth;

    /**
     * tickSpeed is the distance in pixels the message will
     * travel during one tick
     */
    private int tickSpeed;

    /**
     * Font color
     */
    private int color;

    /**
     * Background color
     */
    private int bgColor;

    /**
     * TICK_RATE is the number of milliseconds between ticks
     */
    static final long TICK_RATE = 250;


    //public NewsTicker(String str, Displayable displayable) {
    public FlexiTicker(String str, int width) {
        super("");
        this.width = width;
        height = Font.getDefaultFont().getHeight();
        setupText(str);
        ticking = false;
        color = -1;
        bgColor = -1;
    }

    /**
     * Sets the string to be displayed by this ticker. If this ticker is active
     * and is on the display, it immediately begins showing the new string.
     *
     * @param str string to be set for the <code>Ticker</code>
     * @throws NullPointerException if <code>str</code> is <code>null</code>
     * @see #getString
     */
    public void setString(String str) {
        setupText(str);
    }

    /**
     * Gets the string currently being scrolled by the ticker.
     *
     * @return string of the ticker
     * @see #setString
     */
    public String getString() {
        // SYNC NOTE: return of atomic value, no locking necessary
        return message;
    }

    protected int getMinContentWidth() {
        return width;
    }

    protected int getMinContentHeight() {
        return height;
    }

    protected int getPrefContentWidth(int i) {
        return width;
    }

    protected int getPrefContentHeight(int i) {
        return height;
    }

    protected void paint(Graphics graphics, int i, int i1) {
        if (bgColor >= 0) {
            graphics.setColor(bgColor);
            graphics.fillRect(0, 0, this.width, this.height);
        }
        if (color >= 0)
            graphics.setColor(color);
        
        if (ticking) {
            messageLoc -= tickSpeed;
            graphics.drawString(displayedMessage, messageLoc, 0,
                    Graphics.TOP | Graphics.LEFT);

            // Once the message is completely off the left side of
            // the screen, we reset its location to be the right side
            // of the screen
            if (messageLoc <= -messageWidth) {
                //messageLoc = displayable.getWidth();
                messageLoc = width;
            }
        } else {
            graphics.drawString(displayedMessage, 0, 0,
                    Graphics.TOP | Graphics.LEFT);
        }

    }

    public void commandAction(Command command, Item item) {

    }

    protected void keyPressed(int i) {
        super.keyPressed(i);
    }

    protected boolean traverse(int i, int i1, int i2, int[] ints) {
        boolean ret = super.traverse(i, i1, i2, ints);
        // Set ticking
        ticking = true;
        if (timer != null) {
            timer.cancel();
            timer = null;
            tickerTask = null;
        }
        tickerTask = new TickerTask(this);
        timer = new Timer();
        reset();
        timer.schedule(tickerTask, 0, TICK_RATE);
        return ret;
    }

    protected void traverseOut() {
        super.traverseOut();
        // Stop ticking
        ticking = false;
        if (timer != null) {
            timer.cancel();
            timer = null;
        }
    }

    /**
     * Initialize this Ticker with the given text
     *
     * @param message The text this Ticker will display
     */
    private final void setupText(String message) {
        if (message == null) {
            throw new NullPointerException();
        }

        /*
        * Search the message for linebreak characters, and replace
        * with spaces.
        */
        StringBuffer msg = new StringBuffer(message);
        int offset = 0;
        boolean modified = false;
        while ((offset = message.indexOf('\n', offset)) != -1) {
            msg.setCharAt(offset, ' ');
            offset++;
            modified = true;
        }

        /*
        * Save the original unmodified message so that getString()
        * returns that. If the message is modified because it
        * contains linebreak characters, then set the display message
        * to the modified string.
        */
        this.message = message;
        this.displayedMessage = modified ? msg.toString() : message;

        messageWidth = Font.getDefaultFont().stringWidth(this.displayedMessage);

        if (messageWidth < 5) {
            tickSpeed = messageWidth;
        } else {
            tickSpeed = 5;
        }
        reset();
    }

    void reset() {
        //messageLoc = displayable.getWidth() / 2;
        messageLoc = width / 2;
    }


    public int getTickSpeed() {
        return tickSpeed;
    }

    public void setTickSpeed(int tickSpeed) {
        this.tickSpeed = tickSpeed;
    }


    public int getBgColor() {
        return bgColor;
    }

    public void setBgColor(int bgColor) {
        this.bgColor = bgColor;
    }

    public int getColor() {
        return color;
    }

    public void setColor(int color) {
        this.color = color;
    }

    class TickerTask extends TimerTask {
        FlexiTicker ticker;

        public TickerTask(FlexiTicker ticker) {
            this.ticker = ticker;
        }

        public void run() {
            ticker.repaint();
        }
    }

}

